package com.ydl.ydl_pay.http

import android.annotation.SuppressLint
import com.facebook.stetho.okhttp3.StethoInterceptor
import com.ydl.ydl_pay.BuildConfig
import com.ydl.ydl_pay.bean.params.BaseCommand
import com.ydl.ydl_pay.http.log.LogUtil
import com.ydl.ydl_pay.http.utils.HttpUrlUtils
import okhttp3.FormBody
import okhttp3.MultipartBody
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import okio.Buffer
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.security.KeyManagementException
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import java.security.SecureRandom
import java.security.cert.X509Certificate
import java.util.*
import java.util.concurrent.TimeUnit
import javax.net.ssl.SSLContext
import javax.net.ssl.SSLSocketFactory
import javax.net.ssl.TrustManager
import javax.net.ssl.X509TrustManager

/**
 * @author jiucheng
 * @描述:支付模块的网络请求
 * @Copyright Copyright (c) 2018
 * @Company 壹点灵
 * @date 2019/4/3
 */
class PayRetrofitUtils private constructor() {
    private lateinit var config: HttpConfig


    companion object {
        private const val AUTHORIZATION_NAME = "Authorization"
        private const val SESSION_KEY_DEFAULT_VALUE = "dc59cf294f37d237c1f06240568ffe21"
        private const val YDL = "Ydl "

        private const val UID = "uid"
        private const val FFROM = "ffrom"
        private const val IS_FROM_APP = "isFromApp"//1表示用户版2表示专家版
        private const val OS_BUILD = "osBuild"
        private const val TS = "ts"
        private const val VERSION = "version"
        private const val TOKEN = "accessToken"
        private const val DEFAULT_TIMEOUT = 15
        private const val OS_TYPE = "osType"// 1.ios 2.android

        private fun getInstance(): PayRetrofitUtils {
            return Holder.INSTANCE
        }

        fun setConfig(config: HttpConfig) {
            getInstance().config = config
        }

        // 调用此方法获取 Retrofit 对象
        fun getRetrofit(): Retrofit {
            return getInstance().retrofit
        }


        fun getPostList(crq: BaseCommand): List<FormatText> {
            val formTextList = ArrayList<FormatText>()
            val fields = crq.javaClass.fields
            for (field in fields) {
                try {
                    val accessFlag = field.isAccessible
                    field.isAccessible = true
                    val o = field.get(crq)
                    if (o != null) {
                        val formatText = FormatText(field.name, o.toString())
                        formTextList.add(formatText)
                    }
                    field.isAccessible = accessFlag
                } catch (e: IllegalAccessException) {
                    e.printStackTrace()
                }

            }
            return formTextList
        }

        //获取网络请求表
        fun getMaps(list: List<FormatText>): Map<String, String> {
            val map = HashMap<String, String>()
            val size = list.size
            for (i in 0 until size) {
                val formatText = list[i]
                map[formatText.getmKey()] = formatText.getmValue().toString()
            }
            return map
        }

    }

    private val retrofit: Retrofit by lazy {
        Retrofit.Builder()
                .baseUrl(getBaseJavaUrl())
                .addConverterFactory(GsonConverterFactory.create())
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                .client(getOkHttp())
                .build()
    }

    private fun getBaseJavaUrl(): String {
        return HttpUrlUtils.getBaseUrl(config.isTestEnvironment, true)
    }

    private fun getOkHttp(): OkHttpClient {
        val builder = OkHttpClient.Builder()
        builder.retryOnConnectionFailure(false)

        builder.addInterceptor { chain ->
            val body = chain.request().body()
            val list = ArrayList<FormatText>()
            if (body is FormBody) {
                val formBody = body as FormBody?
                for (i in 0 until formBody!!.size()) {
                    list.add(FormatText(formBody.name(i), formBody.value(i)))
                }
            }
            if (body is MultipartBody) {
                val multipartBody = body as MultipartBody?
                val parts = multipartBody!!.parts()
                for (part in parts) {

                    if ("text/plain; charset=utf-8" == part.body().contentType()!!.toString()) {
                        val headerStr = part.headers()!!.toString()
                        val name = headerStr.split("\"\\n".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()[0].split("=\"".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()[1]
                        val buffer = Buffer()
                        part.body().writeTo(buffer)
                        val value = buffer.readUtf8()
                        list.add(FormatText(name, value))
                    }
                }
            }
            val head = YDL + getSign(list)
            LogUtil.i("RetrofitLog", "head = $head")

            val build = chain.request()
                    .newBuilder()
                    .header(AUTHORIZATION_NAME, head)
                    .addHeader(FFROM, config.ffrom)
                    .addHeader(IS_FROM_APP, config.isFromApp)
                    .addHeader(VERSION, config.version)
                    .addHeader(OS_BUILD, config.osBuild)
                    .addHeader(OS_TYPE, "2")
                    .addHeader(TOKEN, config.accessToken)
                    .addHeader(TS, (System.currentTimeMillis() / 1000).toString())
                    .addHeader(UID, config.uid)
            chain.proceed(build.build())
        }

        if (BuildConfig.DEBUG) {
            builder.addInterceptor(HttpLoggingInterceptor { message ->
                //打印retrofit日志
                LogUtil.i("RetrofitLog", "retrofitBack = $message")
            }.setLevel(HttpLoggingInterceptor.Level.BODY))
            builder.addNetworkInterceptor(StethoInterceptor())
        }
        //使用自定义SSLSocketFactory
        val factory = createSSLSocketFactory()
        if (factory != null) {

            builder.sslSocketFactory(factory)
                    .hostnameVerifier { _, _ -> true }
        }
        builder.connectTimeout(DEFAULT_TIMEOUT.toLong(), TimeUnit.SECONDS)
                .readTimeout(DEFAULT_TIMEOUT.toLong(), TimeUnit.SECONDS)
                .writeTimeout(DEFAULT_TIMEOUT.toLong(), TimeUnit.SECONDS)
        return builder.build()
    }

    //获取请求头签名
    private fun getSign(list: List<FormatText>): String? {
        //将请求参数倒序排列
        Collections.sort(list, SortData())
        val stringBuilder = StringBuilder()
        val size = list.size
        for (i in 0 until size) {
            val formatText = list[i]
            if (i != 0) {
                stringBuilder.append("&")
            }
            stringBuilder.append(String.format("%s=%s", formatText.getmKey(), formatText.getmValue()))
        }
        stringBuilder.append(SESSION_KEY_DEFAULT_VALUE)
        return digestMD5(stringBuilder.toString())
    }

    //用来命名排序规则的
    internal class SortData : Comparator<FormatText> {

        override fun compare(lhs: FormatText, rhs: FormatText): Int {
            val res = lhs.getmKey().compareTo(rhs.getmKey())
            if (res == 0) {
                return 0
            }
            return if (res > 0) -1 else 1

        }
    }

    /**
     * 用于MD5加密
     *
     * @param clear 明文
     * @return 密文
     */
    private fun digestMD5(clear: String): String? {
        val hexDigits = charArrayOf('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f')
        try {
            val strTemp = clear.toByteArray(charset("utf-8"))
            // 使用MD5创建MessageDigest对象
            val mdTemp = MessageDigest.getInstance("MD5")
            mdTemp.update(strTemp)
            val md = mdTemp.digest()
            val j = md.size
            val str = CharArray(j * 2)
            var k = 0
            for (b in md) {
                // 将每个数(int)b进行双字节加密
                str[k++] = hexDigits[b.toInt() shr 4 and 0xf]
                str[k++] = hexDigits[b.toInt() and 0xf]
            }
            return String(str)
        } catch (e: Exception) {
            return null
        }

    }

    /**
     * 默认信任所有的证书
     */
    @SuppressLint("TrustAllX509TrustManager")
    private fun createSSLSocketFactory(): SSLSocketFactory? {

        val xtm = object : X509TrustManager {
            override fun checkClientTrusted(chain: Array<X509Certificate>, authType: String) {}

            override fun checkServerTrusted(chain: Array<X509Certificate>, authType: String) {}

            override fun getAcceptedIssuers(): Array<X509Certificate?> {
                return arrayOfNulls(0)
            }
        }

        val sslContext: SSLContext
        try {
            sslContext = SSLContext.getInstance("SSL")

            sslContext.init(null, arrayOf<TrustManager>(xtm), SecureRandom())

        } catch (e: NoSuchAlgorithmException) {
            e.printStackTrace()
            return null
        } catch (e: KeyManagementException) {
            e.printStackTrace()
            return null
        }

        return sslContext.socketFactory

    }

    private object Holder {
        val INSTANCE = PayRetrofitUtils()
    }
}